/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/
using System.Xml.Serialization;
using DNNContentBuilder.Attributes;

namespace DNNContentBuilder.Configuration
{
	/// <summary>
	/// Summary description for CMSConfiguration.
	/// </summary>
	[XmlRoot("cmsConfiguration")]
	[ConfigurationSection("cmsConfiguration")]
	public class CMSConfiguration
	{
		private bool enableDataCache;
		private int absoluteCacheExpiration;

		/// <summary>
		/// Initializes a new instance of the <see cref="CMSConfiguration"/> class.
		/// </summary>
		public CMSConfiguration()
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="CMSConfiguration"/> class.
		/// </summary>
		/// <param name="enableDataCache"></param>
		/// <param name="absoluteCacheExpiration"></param>
		public CMSConfiguration(
			bool enableDataCache, 
			int absoluteCacheExpiration)
		{
			this.enableDataCache = enableDataCache;
			this.absoluteCacheExpiration = absoluteCacheExpiration;
		}

		/// <summary>
		/// Gets a value indicating whether to enable data caching 
		/// functionality against the database layer.
		/// </summary>
		[XmlElement("enableDataCache", IsNullable=false)]
		public bool EnableDataCache
		{
			get
			{
				return this.enableDataCache;
			}
			set
			{
				this.enableDataCache = value;
			}
		}

		/// <summary>
		/// Gets the absolute cache expiration time in seconds
		/// </summary>
		[XmlElement("absoluteCacheExpiration", IsNullable=false)]
		public int AbsoluteCacheExpiration
		{
			get
			{
				return this.absoluteCacheExpiration;
			}
			set
			{
				this.absoluteCacheExpiration = value;
			}
		}

		public static CMSConfiguration GetConfig()
		{
			return (CMSConfiguration)ConfigurationManager.GetConfiguration(typeof(CMSConfiguration));
		}
	}
}
