/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/
using System.Xml.Serialization;
using DNNContentBuilder.Attributes;

namespace DNNContentBuilder.Configuration
{
	/// <summary>
	/// Represents the root of the configuration graph.
	/// </summary>
	[XmlRoot(ElementName="formletConfiguration")]
	[ConfigurationSection("formletConfiguration")]
	public class FormletConfiguration
	{
		#region Member Variables
		private string controlPath;
		private string configPath;
		private FormletDataCollection formletDefinitions;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="FormletConfiguration"/> class.
		/// </summary>
		public FormletConfiguration()
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="FormletConfiguration"/> class.
		/// </summary>
		/// <param name="controlPath"></param>
		/// <param name="configPath"></param>
		/// <param name="formletDefinitions"></param>
		public FormletConfiguration(
			string controlPath,
			string configPath,
			FormletDataCollection formletDefinitions)
		{
			this.controlPath = controlPath;
			this.configPath  = configPath;
			this.formletDefinitions = formletDefinitions;
		}
		#endregion

		#region Properties
		[XmlElement(ElementName="controlPath")]
		public string ControlPath
		{
			get
			{
				return this.controlPath;
			}
			set
			{
				this.controlPath = value;
			}
		}

		[XmlElement(ElementName="configPath")]
		public string ConfigPath
		{
			get
			{
				return this.configPath;
			}
			set
			{
				this.configPath = value;
			}
		}

		[XmlArray(ElementName="formlets")]
		[XmlArrayItem(ElementName="formlet", Type=typeof(FormletData))]
		public FormletDataCollection Formlets
		{
			get
			{
				return this.formletDefinitions;
			}
			set
			{
				this.formletDefinitions = value;
			}
		}
		#endregion
	}
}
