/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.Data;

namespace DNNContentBuilder 
{
	/// <summary>
	/// Summary description for epContentManagerSettings.
	/// </summary>

	[Serializable]
	public class ContentManagerSettingsInfo 
	{
        
		// Entity member variables for database table epContentManagerSettings
        
		private Int32 mContentManagerSettingsId;
		private Int32 mFormDefinitionId;
		private Int32 mWorkflowId;
		private Int32 mModuleID;
        
		// Calculated Expression member variables for database table epContentManagerSettings
        
        
		// Entity properties for database table epContentManagerSettings
        
		public Int32 ContentManagerSettingsId 
		{
			get { return mContentManagerSettingsId; }
			set { mContentManagerSettingsId = value; }
		}
        
        
		public Int32 FormDefinitionId 
		{
			get { return mFormDefinitionId; }
			set { mFormDefinitionId = value; }
		}
        
        
		public Int32 WorkflowId 
		{
			get { return mWorkflowId; }
			set { mWorkflowId = value; }
		}
        
        
		public Int32 ModuleID 
		{
			get { return mModuleID; }
			set { mModuleID = value; }
		}
	}
    
	public class ContentManagerSettingsController : BaseController
	{
		public ContentManagerSettingsController() : base() {}
		public ContentManagerSettingsController(ConnectionProvider cp) : base(cp) {}
        
		public Int32 Add(ContentManagerSettingsInfo ContentManagerSettingsInfo) 
		{
			return (Int32) base.GetDataProvider().ContentManagerSettingsAdd(ContentManagerSettingsInfo.FormDefinitionId, ContentManagerSettingsInfo.WorkflowId, ContentManagerSettingsInfo.ModuleID);
		}
        
		public void Update(ContentManagerSettingsInfo ContentManagerSettingsInfo) 
		{
			base.GetDataProvider().ContentManagerSettingsUpdate(ContentManagerSettingsInfo.ContentManagerSettingsId, ContentManagerSettingsInfo.FormDefinitionId, ContentManagerSettingsInfo.WorkflowId, ContentManagerSettingsInfo.ModuleID);
		}
        
		public void Delete(Int32 contentManagerSettingsId) 
		{
			base.GetDataProvider().ContentManagerSettingsDelete(contentManagerSettingsId);
		}
        
		public ContentManagerSettingsInfo GetItem(Int32 contentManagerSettingsId) 
		{
			using (IDataReader idr = base.GetDataProvider().ContentManagerSettingsGet(contentManagerSettingsId))
			{
				ContentManagerSettingsInfo infoObject = (ContentManagerSettingsInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(ContentManagerSettingsInfo));
				return infoObject;
			} 
		}
        
		// Relation methods for database table epContentManagerSettings
		public ArrayList GetByFormDefinition(Int32 formDefinitionId) 
		{
			using (IDataReader idr = base.GetDataProvider().ContentManagerSettingsByFormDefinition(formDefinitionId))
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(ContentManagerSettingsInfo));
				return infoList;
			} 
		}
        
		public ArrayList GetByWorkflow(Int32 workflowId) 
		{
			using (IDataReader idr = base.GetDataProvider().ContentManagerSettingsByWorkflow(workflowId))
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(ContentManagerSettingsInfo));
				return infoList;
			} 
		}

		public ContentManagerSettingsInfo GetByModule(Int32 moduleID)
		{
			using (IDataReader idr = base.GetDataProvider().ContentManagerSettingsByModules(moduleID))
			{
				ContentManagerSettingsInfo infoObject = (ContentManagerSettingsInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(ContentManagerSettingsInfo));
				return infoObject;
			} 
		}
	}
}
