/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2007
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Data;
using System.Reflection;

namespace DNNContentBuilder.UI.WebControls.DataBinding
{
	/// <summary>
	/// Summary description for DataTableAdapter.
	/// </summary>
	public class DataTableAdapter
	{
		private DataTable table;

		public DataTableAdapter(System.Collections.IList list, System.Type type)
		{
			this.table = InternalExtractDataTable(list, type);
		}

		/// <summary>
		/// Property Table (DataTable)
		/// </summary>
		public DataTable Table
		{
			get
			{
				return this.table;
			}
			set
			{
				this.table = value;
			}
		}

		/// <summary>
		/// Converts any <see cref="System.Collections.IList"/> derived type
		/// into a DataTable representation.
		/// </summary>
		/// <param name="list">A <see cref="System.Collections.IList"/></param>
		/// <param name="type">The <see cref="System.Type"/> of the items within the list.</param>
		/// <returns></returns>
		private DataTable InternalExtractDataTable(System.Collections.IList list, System.Type type)
		{
			object[] param = null;
			DataTable dt = new DataTable();

			dt.BeginLoadData();

			try
			{
				PropertyInfo[] properties = type.GetProperties(BindingFlags.Instance|BindingFlags.Public);

				foreach (PropertyInfo property in properties)
				{
					dt.Columns.Add(property.Name);
				}

				foreach (object item in list)
				{
					DataRow row = dt.NewRow();

					foreach (PropertyInfo property in properties)
					{
						object value = property.GetValue(item, param); 

						if (value == null)
						{
							value = String.Empty;
						}

						row[property.Name] = value;
					}

					dt.Rows.Add(row);
				}
			}
			finally
			{
				dt.EndLoadData();
			}

			return dt;
		}
	}
}
