/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.Data;

namespace DNNContentBuilder 
{
	[Serializable]
	public class LabelInfo 
	{
		// Entity member variables
        
		private Int32 mLabelId;
		private string mShortName;
		private Int32 mModuleId;
        
        
		// Entity properties
        
		public Int32 ModuleId 
		{
			get { return mModuleId; }
			set { mModuleId = value; }
		}

		public string ShortName
		{
			get { return mShortName; }
			set { mShortName = value; }
		}
        
		public Int32 LabelId 
		{
			get { return mLabelId; }
			set { mLabelId = value; }
		}
	}
    
	public class LabelController : BaseController
	{
		public LabelController() : base() {}
		public LabelController(ConnectionProvider cp) : base(cp) {}

		public Int32 Add(LabelInfo LabelInfo) 
		{
			return (Int32) base.GetDataProvider().LabelAdd(LabelInfo.ShortName, LabelInfo.ModuleId);
		}

		public void Update(LabelInfo LabelInfo) 
		{
			base.GetDataProvider().LabelUpdate(LabelInfo.LabelId, LabelInfo.ShortName, LabelInfo.ModuleId);
		}

		public void Delete(Int32 labelId)
		{
			base.GetDataProvider().LabelDelete(labelId);
		}

		public LabelInfo GetItem(Int32 labelId) 
		{
			using (IDataReader idr = base.GetDataProvider().LabelGet(labelId))
			{
				LabelInfo infoObject = (LabelInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(LabelInfo));
				return infoObject;
			} 
		}

		public ArrayList GetLabelsByModule(Int32 moduleId) 
		{
			using (IDataReader idr = base.GetDataProvider().LabelGetByModule(moduleId))
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(LabelInfo));
				return infoList;
			} 
		}
 
		public ArrayList GetAll() 
		{
			using (IDataReader idr = base.GetDataProvider().LabelGetAll())
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(LabelInfo));
				return infoList;
			} 
		}
	}
}
