/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.Data;

namespace DNNContentBuilder 
{
	[Serializable]
	public class TaxonomyInfo 
	{
        
		// Entity member variables for database table epTaxonomy
        
		private Int32 mTaxonomyId;
		private string mTitle;
		private string mDescription;
		private Int32 mParentId;
        
		// Calculated Expression member variables for database table epTaxonomy
        
        
		// Entity properties for database table epTaxonomy
        
		public Int32 TaxonomyId 
		{
			get { return mTaxonomyId; }
			set { mTaxonomyId = value; }
		}
        
		public string Title 
		{
			get { return mTitle; }
			set { mTitle = value; }
		}
        
		public string Description 
		{
			get { return mDescription; }
			set { mDescription = value; }
		}
        
        
		public Int32 ParentId 
		{
			get { return mParentId; }
			set { mParentId = value; }
		}
        
        
	}
    
	public class TaxonomyController : BaseController
	{
		public TaxonomyController() : base() {}

		public TaxonomyController(ConnectionProvider cp) : base(cp) {}

		public Int32 Add(TaxonomyInfo TaxonomyInfo) 
		{
			return (Int32) base.GetDataProvider().TaxonomyAdd(TaxonomyInfo.Title, TaxonomyInfo.Description, TaxonomyInfo.ParentId);
		}
        
		public void Update(TaxonomyInfo TaxonomyInfo) 
		{
			base.GetDataProvider().TaxonomyUpdate(TaxonomyInfo.TaxonomyId, TaxonomyInfo.Title, TaxonomyInfo.Description, TaxonomyInfo.ParentId);
		}
        
		public void Delete(Int32 taxonomyId) 
		{
			base.GetDataProvider().TaxonomyDelete(taxonomyId);
		}
        
		public TaxonomyInfo GetItem(Int32 taxonomyId) 
		{
			using (IDataReader idr = base.GetDataProvider().TaxonomyGet(taxonomyId))
			{
				TaxonomyInfo infoObject = (TaxonomyInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(TaxonomyInfo));
				return infoObject;
			} 
		}
        
		// Select query methods for database table epTaxonomy
        
		public ArrayList GetTaxonomyRoot() 
		{
			using (IDataReader idr = base.GetDataProvider().TaxonomyGetTaxonomyRoot())
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(TaxonomyInfo));
				return infoList;
			} 
		}
        
		public ArrayList GetTaxonomySubTree(Int32 ParentId) 
		{
			using (IDataReader idr = base.GetDataProvider().TaxonomyGetTaxonomySubTree(ParentId))
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(TaxonomyInfo));
				return infoList;
			} 
		}
        
		public TaxonomyInfo GetByTitle(string taxonomyTitle, Int32 parentId)
		{
			using (IDataReader idr = base.GetDataProvider().TaxonomyByTitle(taxonomyTitle.ToUpper().Trim(), parentId))
			{
				TaxonomyInfo infoObject = (TaxonomyInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(TaxonomyInfo));
				return infoObject;
			}
		}
	}
}
