/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using DNNContentBuilder.Attributes;
using DNNContentBuilder.UserControls.SearchFieldControls;

namespace DNNContentBuilder.UserControls.FieldControls
{
	using System;
	using System.Xml;

	/// <summary>
	///	Summary description for CheckBoxControl.
	/// </summary>
	[FormEngineSearchControl(typeof(DatePickerSearchControl))]
	public class DatePickerControl : BaseFieldControl, IFormlet
	{
		public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/DatePickerControl";

		#region Class Members
		private bool isValid = false;
		private readonly XmlDocument xmlDoc  = new XmlDocument();
		private XmlNode     xmlNode;
		#endregion

		#region Controls
		protected System.Web.UI.WebControls.TextBox txtDate;
		protected System.Web.UI.WebControls.TextBox txtTime;
		protected System.Web.UI.WebControls.HyperLink cmdCalendar;
		protected System.Web.UI.HtmlControls.HtmlGenericControl span;
		#endregion

		#region Event Handlers
		private void Page_Load(object sender, EventArgs e)
		{
			this.cmdCalendar.Text = Commons.GetLocalizedString("label_Calendar", resource);

            cmdCalendar.NavigateUrl = DotNetNuke.Common.Utilities.Calendar.InvokePopupCal(txtDate);
		}
		#endregion

		#region Methods
		public DateTime GetDateTime()
		{
			return DateTime.Parse(this.txtDate.Text + " " + this.txtTime.Text);
		}
		#endregion

		#region IFormlet Members
		
		public bool IsValid
		{
			get
			{
				return this.isValid;
			}
		}

		public string GetValue(string XML)
		{
			xmlDoc.LoadXml(XML);

			xmlNode = xmlDoc.SelectSingleNode("//SystemType");
			xmlNode.InnerText = "DateTime";

			xmlNode = xmlDoc.SelectSingleNode("//Value");

			this.isValid = true;

			try 
			{
				xmlNode.InnerText = this.GetDateTime().ToString("g"); // dd/MM/yyyy hh:mm
			}
			catch 
			{
				xmlNode = xmlDoc.SelectSingleNode("//IsRequired");
			
				if (xmlNode.InnerText.ToLower() == "true") {
					this.isValid = false;
				}
			}

			return xmlDoc.InnerXml;
		}

		public void SetValue(string XML)
		{
			xmlDoc.LoadXml(XML);
			xmlNode = xmlDoc.SelectSingleNode("//Value");

			DateTime date = DateTime.Parse(xmlNode.InnerText);

			this.txtDate.Text = date.Date.ToShortDateString();
			this.txtTime.Text = date.ToString("t");
		}

		public void SetConfig(string XML) 
		{
			xmlDoc.LoadXml(XML);

			xmlNode = xmlDoc.SelectSingleNode("//Showtime");

			if (xmlNode.InnerText.ToUpper() != "TRUE")
			{
				this.txtTime.Visible = true;
				this.txtTime.Text = DateTime.Today.ToString("t");
			} 
			else 
			{
				this.txtTime.Visible = false;
				this.txtTime.Text = "";
			}

			this.txtDate.Text = (DateTime.Today).ToShortDateString();
			
			xmlNode = xmlDoc.SelectSingleNode("//CssClass");

			txtDate.CssClass = xmlNode.InnerText;
			txtTime.CssClass = xmlNode.InnerText;

			span.Attributes["class"] = xmlNode.InnerText;
			
			cmdCalendar.CssClass = txtDate.CssClass;
		}


		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
