/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.Web.UI.WebControls;
using System.Xml;

namespace DNNContentBuilder.UserControls.FieldControls
{
    /// <summary>
	///		Summary description for linker.
	/// </summary>
	public class LinkerDef : System.Web.UI.UserControl, IFormletDef
	{
		public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/LinkerDef";
		public const string resourceengine = "~/DesktopModules/DNNContentBuilder/App_LocalResources/FormSearchControl";

		protected TextBox txtItemsNumber;
		protected RadioButton rb1x1;
		protected RadioButton rb1xM;

		private   XmlNode xmlNode;

		private string primaryKey = "";
		protected TextBox txtParent;

		private readonly XmlDocument xmlDoc = new XmlDocument();

		private void Page_Load(object sender, EventArgs e)
		{
			// Put user code to initialize the page here

			this.rb1x1.Text = Commons.GetLocalizedString("label_1x1", resource);
			this.rb1xM.Text = Commons.GetLocalizedString("label_1xM", resource);


			ArrayList fieldList = (ArrayList)Session["fieldList"];

			primaryKey = "";

			//looking for primary key

			foreach (FieldInfo field in fieldList) 
			{
				if (field.columnProperties != "") 
				{
					xmlDoc.LoadXml(field.columnProperties);

					if (xmlDoc != null) 
					{
						xmlNode = xmlDoc.SelectSingleNode("//PrimaryKey");

						if (xmlNode != null) // the first primary key found will be used as parent in this relationship
						{
							xmlNode = xmlDoc.SelectSingleNode("//SourceField");
							primaryKey = xmlNode.InnerText;
							break;
						}
					}
				}
			}

			if (primaryKey == "") 
			{
				return;
			}

			this.txtParent.Text = primaryKey;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		public void SetXml (string extProperties) 
		{
			if (primaryKey == "") 
			{
				return;
			}

			XmlDocument xml = new XmlDocument();

			XmlNode tempNode;
				
			xml.LoadXml(extProperties);

			tempNode = xml.SelectSingleNode("//LinkedField");

			if (tempNode == null) 
			{
				XmlDocument tempXml = new XmlDocument();

				tempXml.LoadXml(extProperties);

				tempNode = tempXml.SelectSingleNode("//ExtendedProperties");

				XmlDocument cbXml = new XmlDocument();

				cbXml.Load(System.Web.HttpContext.Current.Server.MapPath("~\\DesktopModules\\DNNContentBuilder\\UserControls\\FieldControls\\XML\\LinkerDef.xml"));

				tempNode.InnerXml = cbXml.InnerXml;

				xmlDoc.LoadXml(tempXml.OuterXml);
			} 
			else 
			{
				xmlDoc.LoadXml(extProperties);
			}
			
			tempNode = xmlDoc.SelectSingleNode("//LinkedField");

			if (tempNode.InnerText != "") 
			{
				this.txtParent.Text = tempNode.InnerText;
			} 
			else
			{
				this.txtParent.Text = primaryKey;
			}

			tempNode = xmlDoc.SelectSingleNode("//RelationType");

			if (tempNode.InnerText == "1:1") 
			{
				this.rb1x1.Checked = true;
			} 
			else 
			{
				this.rb1xM.Checked = true;
			}

			tempNode = xmlDoc.SelectSingleNode("//ItemsNumber");

			this.txtItemsNumber.Text = tempNode.InnerText;

			System.Web.UI.Control lastButton = Page.FindControl((string)Session["lastButton"]);

			((ImageButton)lastButton).CommandArgument = xmlDoc.OuterXml;
		}

		public void GetXml (string xml) 
		{
			if (txtParent.Text == "") 
			{
				return;
			}

			XmlDocument tempXml = new XmlDocument();

			XmlNode tempNode;
				
			tempXml.LoadXml(xml);

			if (tempXml != null) 
			{
				xmlNode = tempXml.SelectSingleNode("//SourceField");

				if (xmlNode == null) 
				{
					throw new ApplicationException(Commons.GetLocalizedString("ERR_INVALID_FORMLET", resource)); // TODO: to create another string ERR_INVALID_FIELD
				}
				//Compare this field with primary key
				if (xmlNode.InnerText.Substring(xmlNode.InnerText.IndexOf(".") + 1).ToUpper() != txtParent.Text.Substring(txtParent.Text.IndexOf(".") + 1).ToUpper()) 
				{
					return;
				}
			}

			tempNode = tempXml.SelectSingleNode("//LinkedField");

			tempNode.InnerText = txtParent.Text;

			tempNode = tempXml.SelectSingleNode("//RelationType");

			if (this.rb1x1.Checked) 
			{
				tempNode.InnerText = "1:1";
			} 
			else 
			{
				tempNode.InnerText = "1:M";
			}

			tempNode = tempXml.SelectSingleNode("//ItemsNumber");
			tempNode.InnerText = this.txtItemsNumber.Text;

			tempNode = tempXml.SelectSingleNode("//UniqueId");
			
			System.Web.UI.Control lastButton = Page.FindControl(tempNode.InnerText);
			
			((ImageButton)lastButton).CommandArgument = tempXml.OuterXml;
		}

	}//class
}//namespace
