/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using DNNContentBuilder.UserControls.SearchFieldControls;

namespace DNNContentBuilder.UserControls.FieldControls
{
	using System;
	using System.Xml;
	using Attributes;

	/// <summary>
	///	Summary description for NumberBoxControl.
	/// </summary>
	[FormEngineSearchControl(typeof(NumberSearchControl))]
	public class NumberBoxControl : BaseFieldControl, IFormlet
	{
		private bool isValid = true;

		protected System.Web.UI.WebControls.TextBox txtNumber;

		private void Page_Load(object sender, EventArgs e)
		{
			// Put user code to initialize the page here
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion

/*
		private const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/ResourceEngine";
*/
		
		#region IFormlet Members

		public bool IsValid
		{
			get
			{
				return this.isValid;
			}
		}

		public string GetValue(string XML)
		{
			XmlDocument xmlDoc = new XmlDocument();

			xmlDoc.LoadXml(XML);

			XmlNode xmlNode = xmlDoc.SelectSingleNode("//SystemType");
			xmlNode.InnerText = "Double";
			
			xmlNode = xmlDoc.SelectSingleNode("//Value");

			this.isValid = true;
			
			Double val = 0;
			
			if (txtNumber.Text != "") {
				try {
					val = Double.Parse(txtNumber.Text, System.Globalization.NumberStyles.Number);
				}
				catch (Exception ) {
					this.isValid = false;
					val = 0;
				}
			}
			
			if ((Int32)val == val) {
				xmlNode.InnerText = ((Int32)val).ToString();
			} else {
				xmlNode.InnerText = val.ToString();
			}

			return xmlDoc.InnerXml;
		}

		public void SetValue(string XML)
		{
			XmlDocument xmlDoc = new XmlDocument();

			xmlDoc.LoadXml(XML);

			XmlNode xmlNode = xmlDoc.SelectSingleNode("//Value");

			txtNumber.Text = Commons.DecodeAngleBrackets(xmlNode.InnerText);
		}

		public void SetConfig(string XML)
		{
			XmlDocument xmlDoc = new XmlDocument();

			xmlDoc.LoadXml(XML);

			txtNumber.Attributes["alt"] = ContextUtil.GetAttributeValue(xmlDoc, "Mask", @"{
				type:'number',
				mask:'9999999999',
				stripMask: false
			}");
			
			txtNumber.Text = ContextUtil.GetAttributeValue(xmlDoc, "DefaultValue", "0");
			
			XmlNode xmlNode = xmlDoc.SelectSingleNode("//CssClass");

			txtNumber.CssClass = txtNumber.CssClass.Replace(xmlNode.InnerText, "") + " " + xmlNode.InnerText + " iMask";
		}

		#endregion
	}
}
