/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Text.RegularExpressions;
using System.Web.UI.WebControls;
using System.Xml;

namespace DNNContentBuilder.UserControls.FieldControls
{
    /// <summary>
	///		Summary description for TextBox.
	/// </summary>
	public class TextBoxDef : System.Web.UI.UserControl, IFormletDef
	{
		public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/TextBoxDef";

		protected TextBox txtSize;
		protected TextBox txtMaxLength;
		protected TextBox txtRegExp;
        protected LinkButton btnCheckRegEx;
        protected Label lblRegexCheckResult;
        protected CheckBox chkIgnoreCase;
        protected CheckBox chkECMAScript;
        protected CheckBox chkExplicitCapture;
        protected CheckBox chkMultiLine;
        protected CheckBox chkRightToLeft;
        protected CheckBox chkSingleLine;
        protected CheckBox chkIgnorePatternWhiteSpace;

		private readonly XmlDocument xmlDoc = new XmlDocument();

		private void Page_Load(object sender, EventArgs e)
		{
			// Put user code to initialize the page here
            btnCheckRegEx.Text = Commons.GetLocalizedString("label_btnRegExpValidate", resource);
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

        protected void btnValidateRegEx_Click(object sender, EventArgs e)
        {
            if (this.txtRegExp.Text != String.Empty && this.txtMaxLength.Text != String.Empty)
            {
                RegexOptions options = this.buildRegexOptions();
                if (RegExValidator.ValidateControl(this.txtMaxLength.Text, this.txtRegExp.Text,options))
                {
                    this.lblRegexCheckResult.Text = Commons.GetLocalizedString("label_RegExp_Success", resource);
                }
                else
                {
                    this.lblRegexCheckResult.Text = Commons.GetLocalizedString("label_RegExp_Fail", resource);
                }
            }
        }

        private RegexOptions buildRegexOptions()
        {
            RegexOptions options = new RegexOptions();
            if (this.chkECMAScript.Checked)
            {
                options = RegexOptions.ECMAScript;
            }
            if (this.chkExplicitCapture.Checked)
            {
                options = RegexOptions.ExplicitCapture;
            }
            if (this.chkIgnoreCase.Checked)
            {
                options = RegexOptions.IgnoreCase;
            }
            if (this.chkIgnorePatternWhiteSpace.Checked)
            {
                options = RegexOptions.IgnorePatternWhitespace;
            }
            if (this.chkMultiLine.Checked)
            {
                options = RegexOptions.Multiline;
            }
            if (this.chkRightToLeft.Checked)
            {
                options = RegexOptions.RightToLeft;
            }
            if (this.chkSingleLine.Checked)
            {
                options = RegexOptions.Singleline;
            }
            return options;
        }

		public void SetXml (string extProperties) 
		{
			XmlDocument xml = new XmlDocument();

			XmlNode tempNode;
				
			xml.LoadXml(extProperties);

			tempNode = xml.SelectSingleNode("//MaxLength");

			if (tempNode == null) 
			{
				XmlDocument tempXml = new XmlDocument();

				tempXml.LoadXml(extProperties);

				tempNode = tempXml.SelectSingleNode("//ExtendedProperties");

				XmlDocument cbXml = new XmlDocument();

				cbXml.Load(System.Web.HttpContext.Current.Server.MapPath("~\\DesktopModules\\DNNContentBuilder\\UserControls\\FieldControls\\XML\\TextBoxDef.xml"));

				tempNode.InnerXml = cbXml.InnerXml;

				xmlDoc.LoadXml(tempXml.OuterXml);
			} 
			else 
			{
				xmlDoc.LoadXml(extProperties);
			}
			
			tempNode = xmlDoc.SelectSingleNode("//MaxLength");

			txtMaxLength.Text = tempNode.InnerText;

			tempNode = xmlDoc.SelectSingleNode("//Size");
			txtSize.Text = tempNode.InnerText;

			tempNode = xmlDoc.SelectSingleNode("//RegExp");
			txtRegExp.Text = tempNode.InnerText;

			System.Web.UI.Control lastButton = Page.FindControl((string)Session["lastButton"]);

			((ImageButton)lastButton).CommandArgument = xmlDoc.OuterXml;
		}

		public void GetXml (string xml) 
		{
			XmlDocument tempXml = new XmlDocument();

			XmlNode tempNode;
				
			tempXml.LoadXml(xml);

			tempNode = tempXml.SelectSingleNode("//MaxLength");
			tempNode.InnerText = txtMaxLength.Text;

			tempNode = tempXml.SelectSingleNode("//Size");
			tempNode.InnerText = txtSize.Text;

			tempNode = tempXml.SelectSingleNode("//RegExp");
			tempNode.InnerText = txtRegExp.Text;

			tempNode = tempXml.SelectSingleNode("//UniqueId");
			
			System.Web.UI.Control lastButton = Page.FindControl(tempNode.InnerText);
			
			((ImageButton)lastButton).CommandArgument = tempXml.OuterXml;
		}
	}//class
}//namespace

