/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using DotNetNuke.Common.Utilities;

namespace DNNContentBuilder.UserControls
{
    /// <summary>
    ///		Summary description for FormSettings.
    /// </summary>
    public class FormSettingsControl : DotNetNuke.Entities.Modules.PortalModuleBase
    {
        #region Constants
        public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/FormSettingsControl";
        #endregion

        #region Controls
        protected System.Web.UI.WebControls.LinkButton btCancelSettings;
        protected System.Web.UI.WebControls.LinkButton btUpdate;
        protected System.Web.UI.WebControls.LinkButton btCancel;
        protected System.Web.UI.WebControls.DropDownList ddWorkflowsAvailable;
        protected System.Web.UI.WebControls.ListBox lbFormsAvailable;
        protected System.Web.UI.HtmlControls.HtmlInputHidden txtContentManagerSettingsId;
        #endregion

        #region Member Variables
        private readonly WorkflowController wfController = new WorkflowController();
        private readonly FormDefinitionController fdController = new FormDefinitionController();
        private readonly ContentManagerSettingsController cmsController = new ContentManagerSettingsController();
        private   ContentManagerSettingsInfo cmsInfo = new ContentManagerSettingsInfo();
        #endregion

        #region Event Handlers
        private void Page_Load(object sender, EventArgs e)
        {
            this.btUpdate.Text = Commons.GetLocalizedString("button_Update", resource);
            this.btCancel.Text = Commons.GetLocalizedString("button_Cancel", resource);

            int moduleId = ContextUtil.GetModuleId(this);

            if (moduleId != Null.NullInteger)
            {
                cmsInfo = cmsController.GetByModule(moduleId);
            }

            if (cmsInfo != null) 
            {
                this.txtContentManagerSettingsId.Value = cmsInfo.ContentManagerSettingsId.ToString();
            }

            if (!this.IsPostBack) 
            {
                ArrayList formDefinitions = fdController.GetAllFormDefinition();

                this.lbFormsAvailable.DataSource     = formDefinitions;
                this.lbFormsAvailable.DataTextField  = "Title";
                this.lbFormsAvailable.DataValueField = "FormDefinitionId";
                this.lbFormsAvailable.DataBind();

                if (cmsInfo != null && cmsInfo.FormDefinitionId != Null.NullInteger)
                {
                    lbFormsAvailable.SelectedValue = cmsInfo.FormDefinitionId.ToString();
                }

                ArrayList workflows = wfController.GetAllWorkflows();
			
                this.ddWorkflowsAvailable.DataSource     = workflows;
                this.ddWorkflowsAvailable.DataTextField  = "Title";
                this.ddWorkflowsAvailable.DataValueField = "WorkflowId";
                this.ddWorkflowsAvailable.DataBind();

                System.Web.UI.WebControls.ListItem noWorkflowItem = new System.Web.UI.WebControls.ListItem();

                noWorkflowItem.Text = Commons.GetLocalizedString("label_NoWorkflow", resource);
                noWorkflowItem.Value = "-1";

                ddWorkflowsAvailable.Items.Insert(0, noWorkflowItem);

                if (cmsInfo != null && cmsInfo.FormDefinitionId != Null.NullInteger)
                {
                    ddWorkflowsAvailable.SelectedValue = cmsInfo.WorkflowId.ToString();
                }
            }
        }

        private void btUpdateSettings_Click(object sender, EventArgs e)
        {
            if (this.lbFormsAvailable.SelectedValue.Length == 0) 
            {
                return;
            }

            if (cmsInfo == null)
            {
                // We are creating a new entry
                cmsInfo = new ContentManagerSettingsInfo();
            }

            if (this.txtContentManagerSettingsId.Value.Length == 0) 
            {
                cmsInfo.FormDefinitionId = Int32.Parse(this.lbFormsAvailable.SelectedValue);
                cmsInfo.WorkflowId = Int32.Parse(this.ddWorkflowsAvailable.SelectedValue);
                cmsInfo.ModuleID = this.ModuleId;

                cmsController.Add(cmsInfo);
            } 
            else 
            {
                cmsInfo.FormDefinitionId = Int32.Parse(this.lbFormsAvailable.SelectedValue);
                cmsInfo.WorkflowId = Int32.Parse(this.ddWorkflowsAvailable.SelectedValue);
                cmsInfo.ContentManagerSettingsId = Int32.Parse(this.txtContentManagerSettingsId.Value);
                cmsInfo.ModuleID = this.ModuleId;
	
                cmsController.Update(cmsInfo);
            }

            Response.Redirect(DotNetNuke.Common.Globals.NavigateURL());
        }

        private void btCancel_Click(object sender, EventArgs e)
        {
            Response.Redirect(DotNetNuke.Common.Globals.NavigateURL());
        }
        #endregion

        #region Web Form Designer generated code
        override protected void OnInit(EventArgs e)
        {
            //
            // CODEGEN: This call is required by the ASP.NET Web Form Designer.
            //
            InitializeComponent();
            base.OnInit(e);
        }
		
        /// <summary>
        ///		Required method for Designer support - do not modify
        ///		the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.btUpdate.Click += new System.EventHandler(this.btUpdateSettings_Click);
            this.btCancel.Click += new EventHandler(btCancel_Click);
            this.Load += new System.EventHandler(this.Page_Load);

        }
        #endregion
    }
}