/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.ComponentModel;
using System.Web.UI.WebControls;

namespace DNNContentBuilder.UserControls
{
    /// <summary>
	///		Summary description for TaskListControl.
	/// </summary>
	[DefaultEvent("TaskClicked")]
	public class TaskListControl : System.Web.UI.UserControl
	{
		public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/TaskListControl";

		public string contentType = "";
		protected DataList dlTaskList;
		
/*
		private MetaDataController metaDataController = new MetaDataController();
*/
		private readonly MetaDataTaskOwnerController metaDataTaskOwnerController = new MetaDataTaskOwnerController();
        private readonly MetaDataTaskController taskController = new MetaDataTaskController();

		private MetaDataTaskInfo taskInfo = new MetaDataTaskInfo();
/*
		private MetaDataInfo metaDataInfo = new MetaDataInfo();
*/

		private MetaDataTaskOwnerInfo metaDataTaskOwnerInfo = new MetaDataTaskOwnerInfo();


		#region Events
		[Bindable(false)]
		[Browsable(true)]
		[Category("Behavior")]
		public event TaskClickEventHandler TaskClicked;
		#endregion

		private void Page_Load(object sender, EventArgs e)
		{
			FillTaskList();
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.dlTaskList.ItemCreated += new System.Web.UI.WebControls.DataListItemEventHandler(this.dlTaskList_ItemCreated);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		private void FillTaskList()
		{
			try 
			{
				// setting contentType -> form title
				FormDefinitionController formController = new FormDefinitionController();
			
				ContentManagerSettingsController cmsController = new ContentManagerSettingsController();

				FormDefinitionInfo formInfo = formController.GetItem((((ContentManagerSettingsInfo)cmsController.GetByModule(ContextUtil.GetModuleId(this))).FormDefinitionId));
			
				contentType = formInfo.Title;
			}
			catch (Exception e)
			{
#pragma warning disable 168
				string error = e.Message;
#pragma warning restore 168
				contentType = "";
			}

			dlTaskList.Visible = false;
			
			ArrayList taskList = taskController.TasksByOwnerAndModule(ContextUtil.GetUserId(this), ContextUtil.GetModuleId(this));

			try 
			{
				if (taskList.Count > 0) 
				{
					dlTaskList.DataSource = taskList;
					dlTaskList.DataBind();
  					
					dlTaskList.Visible = true;
				} 
				else 
				{
					dlTaskList.Visible = false;
				}
			}
			catch (Exception e)
			{
#pragma warning disable 168
				string err = e.Message;
#pragma warning restore 168
			}
		}

		private void dlTaskList_ItemCreated(object sender, DataListItemEventArgs e)
		{// This is not fair
			if ((e.Item.ItemType == ListItemType.Item) || (e.Item.ItemType == ListItemType.AlternatingItem))
			{
  				///Task button
  				LinkButton btTaskAction = (LinkButton) e.Item.FindControl("btTaskAction");

  				btTaskAction.Click += this.btTaskAction_Click;
			}
		}

		protected virtual void OnTaskClicked(int metaDataId, int taskId)
		{
			if (this.TaskClicked != null)
			{
				// set task settings
				taskInfo = taskController.GetItem(taskId);

				if (taskInfo.Status == (TaskStatus) '\0') //Not assigned
				{
					taskInfo.Status = TaskStatus.Assigned;
					taskInfo.StartDate = DateTime.Now;
					taskController.Update(taskInfo);
	
					metaDataTaskOwnerInfo = metaDataTaskOwnerController.GetItem(taskId); // Colocar o id da taskowner correto
					metaDataTaskOwnerInfo.OwnerId = ContextUtil.GetUserId(this);
					metaDataTaskOwnerInfo.OwnerType = "U";

					metaDataTaskOwnerController.Update(metaDataTaskOwnerInfo);
 				}

				this.TaskClicked(this, new TaskClickEventArgs(metaDataId, taskId));
			}
		}

 		private void btTaskAction_Click(object sender, EventArgs e)
		{
 		    OnTaskClicked(Int32.Parse(((LinkButton)sender).CommandArgument.Split(',')[0]), Int32.Parse(((LinkButton)sender).CommandArgument.Split(',')[1]));
		}

//		public bool EvaluateContainer(object dataItem) 
//		{
//			if (dataItem !=	null &&	((MetaDataTaskInfo)dataItem).Title == contentType)
//			{
//				return true;
//			}
//
//			return false;
//		}
	}
}
