/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.IO;
using System.Web;
using System.Web.UI;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Tabs;

namespace DNNContentBuilder.UserControls
{
    public enum TemplatePromotion
	{
		Promote	  = 'P',
		Demote    = 'D',
	};
	
	/// <summary>
	///		Summary description.
	/// </summary>
	public class TemplateWorkbenchControl : DotNetNuke.Entities.Modules.PortalModuleBase	{
		public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/TemplateWorkbenchControl";
		
		protected string folderIcon         = "folderIcon.gif";
		protected string fileIcon           = "fileIcon.gif";
	    private   string selectedNode       = "-1";

		protected FunkeLab.MyTreeView TreeView;

		protected TemplateDetailsControl templateDetailsControl;
		protected System.Web.UI.WebControls.PlaceHolder 				plcTemplateDetails;
		protected System.Web.UI.WebControls.ImageButton 				btRefresh;
		protected System.Web.UI.WebControls.ImageButton 				btNewFolder;
		protected System.Web.UI.WebControls.ImageButton 				btDelFolder;
        protected System.Web.UI.WebControls.TextBox 					txtFolderName;
        protected System.Web.UI.WebControls.Label 						lblFolderName;
		protected System.Web.UI.WebControls.RegularExpressionValidator 	regexValidatorFolder;
		protected System.Web.UI.WebControls.RegularExpressionValidator 	regexValidatorTemplate;
		protected System.Web.UI.WebControls.Literal 					ltErrorMessage;
		protected System.Web.UI.WebControls.LinkButton 					btPublishTemplate;
		protected System.Web.UI.WebControls.LinkButton 					btUnPublishTemplate;
	
		protected System.Web.UI.WebControls.ImageButton 				btNewFolderUpdate;
		protected System.Web.UI.WebControls.ImageButton 				btNewFolderCancel;
		
		protected System.Web.UI.WebControls.PlaceHolder 				plcTabStripItems;

		protected System.Web.UI.WebControls.PlaceHolder 				plcNewTemplate;

		protected System.Web.UI.WebControls.ImageButton 				btNewTemplateUpdate;
		protected System.Web.UI.WebControls.ImageButton 				btNewTemplateCancel;
		protected System.Web.UI.WebControls.ImageButton 				btNewTemplate;
		
		protected System.Web.UI.WebControls.Label       				lblTemplateName;
		protected System.Web.UI.WebControls.TextBox     				txtTemplateName;
		
        protected System.Web.UI.WebControls.LinkButton 					btUpdate;
        protected System.Web.UI.WebControls.LinkButton 					btDelete;
        protected System.Web.UI.WebControls.LinkButton 					btCancel;
        protected System.Web.UI.WebControls.LinkButton 					btDetails;
        
        protected System.Web.UI.HtmlControls.HtmlTextArea				txtScript;
        protected System.Web.UI.HtmlControls.HtmlInputHidden 			txtTab;
        protected System.Web.UI.HtmlControls.HtmlSelect                 lbCMSCommands;

		protected string mTemplate;
		protected Int32 mTemplateId;


		public Int32 TemplateId {
			get {
				if (txtTab.Value != "") {
					TemplateDefinitionController tdController = new TemplateDefinitionController();
					TemplateDefinitionInfo tdInfo = tdController.GetItemByName(txtTab.Value.Replace("\\\\","\\").Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), ""));
					if (tdInfo != null) {
						mTemplateId = tdInfo.TemplateDefinitionId;
					} else {
						mTemplateId = -1;
					}
				} else {
					mTemplateId = -1;
				}
				
				return mTemplateId;
			}
		}

		
		#region Web Form Designer generated code

		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load 											+= new EventHandler(this.Page_Load);
			this.btRefresh.Click 								+= new ImageClickEventHandler(btRefresh_Click);
			this.btNewFolder.Click 								+= new ImageClickEventHandler(btNewFolder_Click);
			this.btNewFolderUpdate.Click    					+= new ImageClickEventHandler(btNewFolderUpdate_Click);
			this.btNewFolderCancel.Click 						+= new ImageClickEventHandler(btNewFolderCancel_Click);
			this.btDelFolder.Click         	 					+= new ImageClickEventHandler(btDelFolder_Click);
			this.btUpdate.Click 								+= new EventHandler(this.btUpdate_Click);
			this.btNewTemplateUpdate.Click 						+= new ImageClickEventHandler(this.btNewTemplateUpdate_Click);
			this.btNewTemplateCancel.Click 						+= new ImageClickEventHandler(this.btNewTemplateCancel_Click);
			this.btDelete.Click 								+= new EventHandler(this.btDelete_Click);
			this.btCancel.Click 								+= new EventHandler(this.btCancel_Click);
			this.btNewTemplate.Click							+= new ImageClickEventHandler(this.btNewTemplate_Click);
			this.TreeView.OnNodeClick 							+= new FunkeLab.MyTreeView.OnNodeClickEventHandler(OnNode_Click);
			this.btUnPublishTemplate.Click						+= new System.EventHandler(this.btUnPublishTemplate_Click);
			this.btPublishTemplate.Click 						+= new System.EventHandler(this.btPublishTemplate_Click);
			this.btDetails.Click 								+= new System.EventHandler(this.btDetails_Click);
			this.templateDetailsControl.TemplateDetailsClicked 	+= new TemplateDetailsEventHandler(RestoreButtons);

		}
		#endregion

		private void btPublishTemplate_Click(object sender, EventArgs e) {
			TemplateDeploy(TemplatePromotion.Promote);
		}
		
		private void btUnPublishTemplate_Click(object sender, EventArgs e) {
			TemplateDeploy(TemplatePromotion.Demote);
		}

		private void TemplateDeploy(TemplatePromotion status) {
			TemplateDefinitionController tdController = new TemplateDefinitionController();
			
			TemplateDefinitionInfo tdInfo = tdController.GetItemByName(txtTab.Value.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), ""));
			
			if (tdInfo == null) {
				ShowError(Commons.GetLocalizedString("label_TemplateNotSaved", resource), "");
				return;          
			}
			
			ArrayList list = new ArrayList();
			
			ArrayList tabs = new TabController().GetTabs(ContextUtil.GetPortalId(this));
			

			foreach (TabInfo tabInfo in tabs)
			{
				// Build the first level items and proceed from there
				if (tabInfo.Level == 0)
				{
					if (!tabInfo.IsAdminTab && !tabInfo.IsDeleted && !tabInfo.IsSuperTab) {
						list = CreateTabNode(tabInfo.TabID, tabInfo.TabName, list);
						
						ArrayList modules = new DotNetNuke.Entities.Modules.ModuleController().GetPortalTabModules(ContextUtil.GetPortalId(this), tabInfo.TabID);
						
						if (modules != null) {

							foreach (DotNetNuke.Entities.Modules.ModuleInfo module in modules) {
								if (module.ModuleName.ToUpper().Trim() == "CONTENT VIEWER") {
									list.Add(new Pair(tabInfo.TabName + module.ModuleTitle, module.ModuleID.ToString()));
								}
							}
						}
					}
				}
			}
			
			SortedList orderedList = new SortedList();

			Hashtable hash = new Hashtable();

			foreach (Pair pair in list) {
				if (!hash.ContainsValue(pair.Value)) {
					orderedList.Add(pair.Value, pair);
					hash.Add(pair.Value, pair.Value);
				}
			}
			
			ContentViewerSettingsController cvsController = new ContentViewerSettingsController();
			
			foreach (Pair pair in orderedList.Values) {
				ArrayList cvs = cvsController.GetByModule(Int32.Parse(pair.Value));

				if (cvs != null && cvs.Count > 0) {
					ContentViewerSettingsInfo cvsInfo = (ContentViewerSettingsInfo)cvs[0];
					if (cvsInfo != null) {
						if (cvsInfo.TemplateDefinitionId != Null.NullInteger) {
							TemplateDefinitionInfo tdi = tdController.GetItem(cvsInfo.TemplateDefinitionId);
							if (tdi != null) {
								if (tdi.TemplateDefinitionId == this.TemplateId) {
									if (status == TemplatePromotion.Demote) {
										try 
										{
											File.Delete(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates\$Live\" + cvsInfo.ModuleID.ToString() + ".ascx"));
										}
										catch (Exception err)
										{
											ShowError(err.Message, resource);
											return;          
										}
						
										this.btPublishTemplate.Visible   = true;
										this.btUnPublishTemplate.Visible = false; 
									} else {
										try {
											string path = HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates\$Live\" + cvsInfo.ModuleID.ToString() + ".ascx");
											
											using (FileStream fs = File.Open(path, FileMode.Create, FileAccess.Write,FileShare.None))
											{
												byte[] buffer = System.Text.Encoding.UTF8.GetBytes(HttpUtility.HtmlDecode(tdi.Body));

												fs.Write(buffer,0,buffer.Length);

												fs.Flush();
											}
											this.btUnPublishTemplate.Visible = true;
										}
										catch (Exception err) {
											ShowError(err.Message, resource);
											return;          
										} 
									}
								}
							}
						}
					}
				}
			}
		}
		
		private ArrayList CreateTabNode(int parentId, string path, ArrayList list) {
			ArrayList tabs = new TabController().GetTabsByParentId(parentId);

			foreach (TabInfo tabInfo in tabs)
			{
				if (!tabInfo.IsAdminTab && !tabInfo.IsDeleted && !tabInfo.IsSuperTab) {
					path = path + tabInfo.TabName;
					
					list = CreateTabNode(tabInfo.TabID, path, list);

					ArrayList modules = new DotNetNuke.Entities.Modules.ModuleController().GetPortalTabModules(ContextUtil.GetPortalId(this), tabInfo.TabID);
					
					if (modules != null) {
						foreach (DotNetNuke.Entities.Modules.ModuleInfo module in modules) {
							if (module.ModuleName.ToUpper().Trim() == "CONTENT VIEWER") {
								list.Add(new Pair(path + module.ModuleTitle, module.ModuleID.ToString()));
							}
						}
					}
				}
			}

			return list;
		}
		
		private void SetDelFolderVisibility() {
			ArrayList list = (ArrayList)Session["OpenedDocuments"];
			
			if (list != null) {
				if (list.Count > 0) {
					btDelFolder.Visible = false;
					if (txtTab.Value == "") {
						foreach (System.Web.UI.WebControls.Button button in list) {
							if (button.BackColor == System.Drawing.Color.White) {
								txtTab.Value = button.CommandName;
								txtScript.Value = button.CommandArgument;
								break;
							}
						}
					}
				} else {
					btDelFolder.Visible = true;
				}
			} else {
				btDelFolder.Visible = true;
			}
		}
		
		private void Page_Load(object sender, EventArgs e)
		{
			SetDelFolderVisibility();

			this.ltErrorMessage.Text = "";

			if (txtTab.Value == ""  || txtTab.Value == null) {
				this.plcTabStripItems.Controls.Clear();
				Session.Remove("OpenedDocuments");
				txtScript.Value = "";
				ChangeThingsVisibility(false);
				txtTab.Value = "";
			}
			
			ShowOpenDocumentsStrip();
			
			//Coding can be a time consuming task...
			Session.Timeout += 2; 
			
			this.lblFolderName.Text           		= Commons.GetLocalizedString("label_FolderName", resource);
			this.btNewFolderUpdate.AlternateText 	= Commons.GetLocalizedString("button_CreateFolder", resource);
			this.btDelFolder.AlternateText    		= Commons.GetLocalizedString("button_DeleteFolder", resource);
			this.btNewFolder.AlternateText    		= Commons.GetLocalizedString("button_NewFolder", resource);
			
			if (!this.IsPostBack) {
				this.btNewFolderUpdate.Visible = false;
				this.btNewFolderCancel.Visible = false;
				this.txtFolderName.Visible = false;
				this.btDelete.Visible = false;
				this.btDetails.Visible = false;
				this.btPublishTemplate.Visible = false;
				this.btUnPublishTemplate.Visible = false;
			} else {
				this.btDelete.Attributes.Add("onClick", "javascript:return confirm('" + Commons.GetLocalizedString("label_ConfirmDelete", resource) + "')");
				this.btCancel.Attributes.Add("onClick", "javascript:return confirm('" + Commons.GetLocalizedString("label_ConfirmCancel", resource) + "')");
//				SetSelectedNode();
			}

			BuildTreeview();

			this.btDetails.Text 					= Commons.GetLocalizedString("button_Details", resource);
			this.btCancel.Text  					= Commons.GetLocalizedString("button_Cancel", resource);
			this.btDelete.Text  					= Commons.GetLocalizedString("button_Delete", resource);
			this.btUpdate.Text  					= Commons.GetLocalizedString("button_Update", resource);
			this.lblTemplateName.Text 				= Commons.GetLocalizedString("label_TemplateName", resource);
			this.btPublishTemplate.Text 			= Commons.GetLocalizedString("button_PublishTemplate", resource);
			this.btUnPublishTemplate.Text 			= Commons.GetLocalizedString("button_UnPublishTemplate", resource);
			
			if (Request["hidAction"] != "" && Request["hidAction"] != null) {
				this.Document_Click(Request.Form.Get("hidAction"));
			}

			lbCMSCommands.Attributes.Add("onchange", "javascript: insertCMSCommand(this);");
		}
		
		private void BuildTreeview() {
			TreeView.Items.Clear();
				
			string path = Page.ResolveUrl("~/DeskTopModules/DNNContentBuilder/images/");

			TreeView.FontSize = 2;

			FunkeLab.MyTreeView.ImagesFolder = path;
			FunkeLab.MyTreeViewItem item = new FunkeLab.MyTreeViewItem();
			
			item.Text  = "Templates";
			item.Open  = true;
			item.Image = Page.ResolveUrl("~/DeskTopModules/DNNContentBuilder/images/") + folderIcon;

			path =  HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates");
			
			item.Value = path;
			
			if (selectedNode == path) {
				item.Selected = true; 
			} else {
				item.Selected = false;
			}
			
			item.Selector = FunkeLab.MyTreeViewItem.SelectionMode.RadioButton;
			
			this.TreeView.Items.Add(item);

			CreateNode(item.Value, item, path);

			TreeView.OpenAll();
		}
		
#pragma warning disable 168
		private FunkeLab.MyTreeViewItem CreateNode(string parent, FunkeLab.MyTreeViewItem localItem, string path) {
#pragma warning restore 168
			string[] folders = Directory.GetDirectories(path);
			
			foreach (string folder in folders) {
				if (folder.IndexOf("$") == -1) {
					FunkeLab.MyTreeViewItem item = new FunkeLab.MyTreeViewItem();

					item.Text  = folder.Substring(folder.LastIndexOf("\\") + 1);
					item.Value = folder;
				
					if (item.Value == selectedNode) {
						item.Selected = true;
					}
				
					item.Selector = FunkeLab.MyTreeViewItem.SelectionMode.RadioButton;
					item.Image = Page.ResolveUrl("~/DeskTopModules/DNNContentBuilder/images/") + folderIcon;

					localItem.Items.Add(CreateNode(folder, item, folder));
				}
			}
			
			string[] files = Directory.GetFiles(path, "*.ascx");
			
			SortedList sortedList = new SortedList();

			foreach (string file in files) {
				if (file.IndexOf("$") == -1) {
					sortedList.Add(file, file);
				}
			}
		
			foreach (string file in sortedList.Keys) {
				FunkeLab.MyTreeViewItem myItem = new FunkeLab.MyTreeViewItem();
				
				myItem.Text  = file.Substring(file.LastIndexOf("\\") + 1);
				myItem.Value = file;
				myItem.Image = Page.ResolveUrl("~/DeskTopModules/DNNContentBuilder/images/") + fileIcon;

				localItem.Items.Add(myItem);
			}
			
			return localItem;
		}

		private void btUpdate_Click(object sender, EventArgs e) {
			SaveTemplate();
		}
		
		private void btNewTemplate_Click(object sender,  ImageClickEventArgs e) {
			btNewFolderCancel_Click(sender, e);
			
			this.plcTabStripItems.Visible = false;
			this.plcNewTemplate.Visible = true;
			this.btNewTemplate.Visible = false;
			this.lbCMSCommands.Visible = false;
			
			SaveCurrentTemplate();
			
			txtScript.Value = "";
		}
		
		private void btNewTemplateUpdate_Click(object sender,  ImageClickEventArgs e) {
			regexValidatorTemplate.ErrorMessage = String.Format("<image src='{0}' border=0 alt='{1}'/>", Page.ResolveUrl("~/DesktopModules/DNNContentBuilder/images/icon_erro.gif"), Commons.GetLocalizedString("ERR_INVALID_FORM_DATA", resource));
			regexValidatorTemplate.ValidationExpression = @"[a-zA-Z0-9\s\-]*";
			regexValidatorTemplate.Enabled = true;

			if (!regexValidatorTemplate.IsValid) {
				return;
			}

			string radio = TreeView.UniqueID.Replace("TreeView", "");

			string path = Page.Request.Form.Get(radio);
			
			if (path == "" || path == null) {
				ShowError(Commons.GetLocalizedString("label_FolderNotSelected", resource));
				return;
			}

			try {
				using (FileStream fs = File.Open(path + "\\" + this.txtTemplateName.Text + ".ascx", FileMode.Create, FileAccess.Write,FileShare.None))
				{
					byte[] buffer = System.Text.Encoding.UTF8.GetBytes(HttpUtility.HtmlDecode(txtScript.Value));

					fs.Write(buffer,0,buffer.Length);

					fs.Flush();
				}
			}
			catch (Exception err) {
				ShowError(Commons.GetLocalizedString("label_FileCannotBeCreated", resource), err.Message);
				return;
			}
			
			this.BuildTreeview();
			
			this.plcTabStripItems.Visible = true;
			this.plcNewTemplate.Visible = false;
			this.btNewTemplate.Visible = true;
			this.lbCMSCommands.Visible = true;
			
			RestoreCurrentTemplate();
		}

		private void btNewTemplateCancel_Click(object sender,  ImageClickEventArgs e) {
			this.plcTabStripItems.Visible = true;
			this.plcNewTemplate.Visible = false;
			this.btNewTemplate.Visible = true;
			this.lbCMSCommands.Visible = true;
			
			RestoreCurrentTemplate();
		}

		private void SaveCurrentTemplate() {
			if (txtScript.Visible == false) {
				return;
			}
			
			string script = txtScript.Value;

			ArrayList list = (ArrayList)Session["OpenedDocuments"];
			
			try {
				foreach (System.Web.UI.WebControls.Button button in list) {
					if (Normalize(button.CommandName) == Normalize(txtTab.Value)) {
						button.CommandArgument = script;
						break;
					}
				}			
			}
			catch (Exception e) {
				Commons.LogError(e, "DNN Content Builder: Template Workbench");
				
				Response.Redirect(DotNetNuke.Common.Globals.NavigateURL(), true);
			}
			
			Session["OpenedDocuments"] = list;
		}

		private void RestoreCurrentTemplate() {
			if (!txtScript.Visible) {
				return;
			}
			
			ArrayList list = (ArrayList)Session["OpenedDocuments"];
			
			foreach (System.Web.UI.WebControls.Button button in list) {
				if (Normalize(button.CommandName) == Normalize(txtTab.Value)) {
					txtScript.Value = button.CommandArgument;
					break;
				}
			}			
		}
		
		private void SaveTemplate() {
			string radio = TreeView.UniqueID.Replace("TreeView", "");

			string path = Page.Request.Form.Get(radio);
			
			if (path != null && path != "") {
				path = path + txtTab.Value.Substring(txtTab.Value.LastIndexOf("\\"));
			} else {
				path = txtTab.Value;
			}
			
			try {
				using (FileStream fs = File.Open(path, FileMode.Create, FileAccess.Write,FileShare.None))
				using (TemplateDefinitionController tdController = new TemplateDefinitionController())
				{
					byte[] buffer = System.Text.Encoding.UTF8.GetBytes(HttpUtility.HtmlDecode(txtScript.Value));

					fs.Write(buffer,0,buffer.Length);

					fs.Flush();
					
					TemplateDefinitionInfo tdInfo;

					path = path.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), "");

					tdInfo = tdController.GetItemByName(path);

					// check if template exist in the old path schema
					if (tdInfo == null) {
						tdInfo = tdController.GetItemByName(path.Substring(1));
					}

					if (tdInfo != null) {
						tdInfo.TemplateName = path;
						tdInfo.Body = txtScript.Value;
						tdController.Update(tdInfo);
					} else {
						tdInfo = new TemplateDefinitionInfo();
						tdInfo.TemplateName = path;
						tdInfo.Body = txtScript.Value;
						tdController.Add(tdInfo);
					}
				}
			}
			catch (Exception err) {
				ShowError(Commons.GetLocalizedString("label_FileCannotBeCreated", resource), err.Message);
				return;
			}
			
			this.BuildTreeview();
			
			this.plcTabStripItems.Visible = true;
			this.plcNewTemplate.Visible = false;
			this.btNewTemplate.Visible = true;
			
			SaveCurrentTemplate();
			
			object sender = new object();
			EventArgs e = new EventArgs();
			
			btCancel_Click(sender, e);
		}
		
		private void btCancel_Click(object sender, EventArgs e) {
			string currentDocument = txtTab.Value;

			this.plcTabStripItems.Controls.Clear();
			
			ArrayList list = (ArrayList)Session["OpenedDocuments"];
			
			int index = -1;
			
			if (list != null) {
				foreach (System.Web.UI.WebControls.Button button in list) {
					button.Attributes.Add("style", "background-color:LightCyan; height: 1.5em; width: 9em;");
					if (Normalize(button.CommandName) == Normalize(currentDocument)) {
						index = list.IndexOf(button);
					} else {
						this.plcTabStripItems.Controls.Add(button);
					}
				}
				
				if (index > -1) {
					list.RemoveAt(index);
				}
			
				if (list.Count > 0) {
					((System.Web.UI.WebControls.Button)list[0]).Attributes.Add("style", "background-color:White; height: 1.8em; width: 9em;");
					Session["OpenedDocuments"] = list;
					txtTab.Value = ((System.Web.UI.WebControls.Button)list[0]).CommandName;
					txtScript.Value = ((System.Web.UI.WebControls.Button)list[0]).CommandArgument;
				} else {
					Session.Remove("OpenedDocuments");
					txtScript.Value = "";
					ChangeThingsVisibility(false);
					txtTab.Value = "";
				}
			}
			
			SetDelFolderVisibility();
		}

		private void btDelete_Click(object sender, EventArgs e) {
			try {
				using (TemplateDefinitionController tdController = new TemplateDefinitionController())
				{
					TemplateDefinitionInfo tdInfo = tdController.GetItemByName(txtTab.Value.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), ""));
				
					tdController.Delete(tdInfo.TemplateDefinitionId);
				}
			}
			catch (Exception ) {
				ShowError(Commons.GetLocalizedString("label_TemplateHasMultipleAssociations", resource));
				return;
			}
				
				
#pragma warning disable 168
			string fileToDelete = txtTab.Value;
#pragma warning restore 168
			
			try {
				File.Delete(txtTab.Value);
			}
	
			catch (Exception err) 
			{
				ShowError(err.Message, "");
				return;
			}
			
			btCancel_Click(sender, e);

			BuildTreeview();
		}

		private void btDetails_Click(object sender, EventArgs e) {
			btUpdate.Visible = false;
			btCancel.Visible = false;
			btDelete.Visible = false;
			btDetails.Visible = false;
			btPublishTemplate.Visible = false;
			btUnPublishTemplate.Visible = false;
			
			UserControl control = (UserControl)plcTemplateDetails.FindControl("templateDetailsControl");
			
			if (control != null) {
				((TemplateDetailsControl)control).ClearItems();
			}
			
			TemplateDefinitionController tdController = new TemplateDefinitionController();
			
			tdController.GetItemByName(txtTab.Value.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), ""));
			
			plcTemplateDetails.Visible = true;
		}

		private void RestoreButtons(object sender, TemplateDetailsEventArgs  e) {
			this.ChangeThingsVisibility(true);

			plcTemplateDetails.Visible = false;
		}
		
		private void btDelFolder_Click(object sender, ImageClickEventArgs e) {
			string radio = TreeView.UniqueID.Replace("TreeView", "");

			if (Page.Request.Form.Get(radio) == null)
			{
				return;
			}

			selectedNode = Page.Request.Form.Get(radio);
			
			string path =  HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates");
			
			if (selectedNode == path) {
				return;
			}
			
			try {
				Directory.Delete(selectedNode);
			}
			catch (Exception err) {
				ShowError(Commons.GetLocalizedString("label_FolderCannotBeDeleted", resource), err.Message);
				return;
			}
			
			// since asp.net recycle the app after a directory removal, the following statement is necessary (sob)
			Response.Redirect(DotNetNuke.Common.Globals.NavigateURL(), true);
		}
		
		private void btNewFolder_Click(object sender, EventArgs e) {
			this.btNewFolderUpdate.Visible = true;
			this.btNewFolderCancel.Visible = true;
			this.txtFolderName.Visible = true;
			this.lblFolderName.Visible = true;

			this.btRefresh.Visible = false;
			this.btNewFolder.Visible = false;
			this.btDelFolder.Visible = false;
		}
		
		private void btNewFolderUpdate_Click(object sender, EventArgs e) {
			
			regexValidatorFolder.ErrorMessage = String.Format("<image src='{0}' border=0 alt='{1}'/>", Page.ResolveUrl("~/DesktopModules/DNNContentBuilder/images/icon_erro.gif"), Commons.GetLocalizedString("ERR_INVALID_FORM_DATA", resource));
			regexValidatorFolder.ValidationExpression = @"[a-zA-Z0-9\s\-]*";
			regexValidatorFolder.Enabled = true;
			
			if (!regexValidatorFolder.IsValid) {
				return;
			}
			
			string radio = TreeView.UniqueID.Replace("TreeView", "");

			if (Page.Request.Form.Get(radio) == null || txtFolderName.Text.Trim() == "")
			{
				return;
			}
			
			selectedNode = Page.Request.Form.Get(radio);
			
			try {
				Directory.CreateDirectory(selectedNode + "\\" + this.txtFolderName.Text);
			}
			catch(Exception err) {
				ShowError(err.Message, "");
				return;
			}
			
			this.BuildTreeview();
			
			txtFolderName.Text = "";
			
			btNewFolderCancel_Click(sender, e);
		}
		
		private void btNewFolderCancel_Click(object sender, EventArgs e) {
			this.btNewFolderUpdate.Visible = false;
			this.btNewFolderCancel.Visible = false;
			this.txtFolderName.Visible = false;
			this.lblFolderName.Visible = false;
			
			this.btRefresh.Visible = true;
			this.btNewFolder.Visible = true;
		}

		private void btRefresh_Click(object sender, EventArgs e) {
			BuildTreeview();
		}
		
		protected void ShowOpenDocumentsStrip() {
			ArrayList list = (ArrayList)Session["OpenedDocuments"];
			
			if (list == null) {
				return;
			}
			
			this.plcTabStripItems.Controls.Clear();
			
			foreach (System.Web.UI.WebControls.Button button in list) {
				this.plcTabStripItems.Controls.Add(button);
			}
		}

		protected void Document_Click(string documentName) {
            //string script = txtScript.Value;

			plcTemplateDetails.Visible = false;

			SaveCurrentTemplate();
			
			ArrayList list = (ArrayList)Session["OpenedDocuments"];
			
			this.plcTabStripItems.Controls.Clear();
			
			foreach (System.Web.UI.WebControls.Button button in list) {
				button.Attributes.Clear();
				
				if (Normalize(button.CommandName) == Normalize(documentName)) {
					txtScript.Value = button.CommandArgument;
					button.Attributes.Add("style", "background-color:White; height: 1.8em; width: 9em;");
					txtTab.Value = button.CommandName;
				} else {
					button.Attributes.Add("style", "background-color:LightCyan; height: 1.5em; width: 9em;");
				}

				this.plcTabStripItems.Controls.Add(button);
			}
			
			ChangeThingsVisibility(true);
		}
	
		protected void OnNode_Click(ref FunkeLab.MyTreeViewItem item)
		{
			// Ignore click on folders
			if (item.Selector == FunkeLab.MyTreeViewItem.SelectionMode.RadioButton) {
				return;
			}

			this.btDelFolder.Visible = false;
			this.plcTemplateDetails.Visible = false;
			
			//check if template is opened
			//open file, create tab, update script, show script
		
			bool isOpened = false;

			string script = "";
			
			ArrayList list = (ArrayList)Session["OpenedDocuments"];
			
			// save current script in memory
			
			if (txtTab.Value != "") {
				foreach (System.Web.UI.WebControls.Button button in list) {
					if (Normalize(button.CommandName) == Normalize(txtTab.Value)) {
						button.CommandArgument = txtScript.Value;
						break;
					}
				}
			}
			
			// check if clicked template is opened
			if (list != null) {
				this.plcTabStripItems.Controls.Clear();
			
				foreach (System.Web.UI.WebControls.Button button in list) {
					button.Attributes.Add("style", "background-color:LightCyan; height: 1.5em; width: 9em;");
					if (Normalize(button.CommandName) == Normalize(item.Value)) {
						isOpened = true;
						txtTab.Value = button.CommandName;
						script = button.CommandArgument;
						button.Attributes.Add("style", "background-color:White; height: 1.8em; width: 9em;");
					}
					
					this.plcTabStripItems.Controls.Add(button);
				}	
			} else {
				list = new ArrayList();
			}
			// Se no est aberto...
			if (!isOpened) {
				using (TemplateDefinitionController tdController = new TemplateDefinitionController()) {
					TemplateDefinitionInfo tdInfo;

					string path = item.Value.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), "");
					//Check if template exists in the new path schema
					tdInfo = tdController.GetItemByName(path);
					
					//Check if template exists in the old path schema
					if (tdInfo == null) {
						path = item.Value.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), "");
						tdInfo = tdController.GetItemByName(path.Substring(1));
					}
					// if template exists only in the directory, create it on the DB
				
					if (tdInfo == null) {
						tdInfo = new TemplateDefinitionInfo();
					
						if (File.Exists(item.Value)) 
						{
							using(StreamReader source = File.OpenText(item.Value)) 
							{
								script = source.ReadToEnd();
							}
						}

						tdInfo.Body             = script;
						tdInfo.TemplateName     = item.Value.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), "");
					
						tdController.Add(tdInfo);
					} else {
						path = item.Value.Replace(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates"), "");
					
						script = tdInfo.Body;
					}
				
					// crie e atribua o script ao hidden, tabstrip e ao editor
					System.Web.UI.WebControls.Button button = new System.Web.UI.WebControls.Button();
				
					button.CommandName = item.Value.Replace("\\", "\\\\");
					button.Text = (item.Value.Substring(item.Value.LastIndexOf("\\") + 1)).Replace(".ascx","");
					button.Attributes.Add("style", "background-color:White; height: 1.8em; width: 9em;");
					button.CommandArgument = script;
					button.BorderWidth = System.Web.UI.WebControls.Unit.Parse("1");
					button.BorderStyle = System.Web.UI.WebControls.BorderStyle.Solid;
					button.Font.Size = System.Web.UI.WebControls.FontUnit.XXSmall;
				
					//this is weird, but ASP.NET is weirdest!
					button.OnClientClick = "document.getElementById('hidAction').value='" + button.CommandName + "';";

					list.Add(button);

					this.plcTabStripItems.Controls.Add(button);
				}
			}
			
			txtTab.Value = item.Value;
			
			txtScript.Value = script;

			ChangeThingsVisibility(true);
			
			Session["OpenedDocuments"] = list;
		}
		
		private void ChangeThingsVisibility(bool visible) {
			lbCMSCommands.Visible = visible;

			txtScript.Visible = visible;

			btUpdate.Visible = visible;
			btDetails.Visible = visible;
			btDelete.Visible = visible;
			btCancel.Visible = visible;
			
			btPublishTemplate.Visible = false;
			btUnPublishTemplate.Visible = false;
			
			if (visible) {
				ContentViewerSettingsController cvsController = new ContentViewerSettingsController();
				
				ArrayList list = cvsController.GetByTemplateDefinition(this.TemplateId);
				
				if (list != null && list.Count > 0) {
					foreach (ContentViewerSettingsInfo cvsInfo in list) {
						this.btPublishTemplate.Visible = true;
						if	(File.Exists(HttpContext.Current.Server.MapPath(@"~\DesktopModules\DNNContentBuilder\Templates\$Live\" + cvsInfo.ModuleID.ToString() + ".ascx"))) {
							this.btUnPublishTemplate.Visible = true;
						}
					}
				}
			}
		}

		private string Normalize(string str) {
			string toReturn = str.Replace("\\","").Replace(":","").Replace(".","");
			
			return toReturn;
		}
		
		private void ShowError(string message, string error) {
			ltErrorMessage.Visible = true;
			ltErrorMessage.Text = @"
			<script language='javascript'>
			<!-- 
			    alert('{0}') 
			-->
			</script>
			";
			ltErrorMessage.Text = String.Format(ltErrorMessage.Text, message);
			ltErrorMessage.Text = String.Format(ltErrorMessage.Text, error).Replace("\r","").Replace("\n","");
			ltErrorMessage.Text = ltErrorMessage.Text.Replace("<!--","\n<!--\n").Replace("-->","\n-->\n");
		}
		
		private void ShowError(string message) {
			ShowError(message, "");
		}
	}
}
