/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Web;
using System.Web.UI;
using DNNContentBuilder.UserControls;

namespace DNNContentBuilder.Attributes
{
	/// <summary>
	/// Use this attribute to decorate <see cref="IFormlet"/> derived controls for the 
	/// <see cref="FormSearchControl"/> class.
	/// </summary>
	[AttributeUsage(AttributeTargets.Class)]
	public sealed class FormEngineSearchControlAttribute : System.Attribute
	{
		#region Class Members
		private const string SearchControlsPath = @"~/DesktopModules/DNNContentBuilder/UserControls/SearchFieldControls/";
		private Type formSearchControl;
		#endregion

		#region Constants
		private const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/ResourceEngine";
		#endregion

		#region Constructors
		public FormEngineSearchControlAttribute(System.Type formSearchControl)
		{
			this.formSearchControl = formSearchControl;
		}
		#endregion

		#region Properties
		public Type FormSearchControl
		{
			get
			{
				return this.formSearchControl;
			}
		}

		public string ControlPath
		{
			get
			{
				if (!(HttpContext.Current.Handler is System.Web.UI.Page))
				{
					throw new ApplicationException(String.Format(Commons.GetLocalizedString("INVALID_HTTP_CONTEXT", resource), this.GetType().Name));
				}

				return ((Page)HttpContext.Current.Handler).ResolveUrl(SearchControlsPath + this.FormSearchControl.Name + ".ascx");
			}
		}
		#endregion
	}
}
