/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/
using System;
using System.Data;

namespace DNNContentBuilder
{
	/// <summary>
	/// Summary description for BaseController.
	/// </summary>
	public abstract class BaseController : System.IDisposable
	{
		private ConnectionProvider connectionProvider = null;
		private bool disposed = false;

		public BaseController()
		{
			// Nothing for now.
		}

		public BaseController(ConnectionProvider cp)
		{
			this.connectionProvider = cp;
		}

		public void BeginTransaction()
		{
			this.BeginTransaction(IsolationLevel.Serializable);
		}

		public void BeginTransaction(IsolationLevel isolationLevel)
		{
			connectionProvider.BeginTransaction(isolationLevel);
		}

		public void Commit()
		{
				connectionProvider.Commit();
		}
		
		public void Rollback()
		{
			connectionProvider.Rollback();
		}

		public DataProvider GetDataProvider()
		{
			if (connectionProvider == null)
			{
				return DataProvider.Instance();
			}

			return DataProvider.Instance(connectionProvider);
		}

		#region IDisposable Members

		void IDisposable.Dispose()
		{
			if (!disposed) 
			{
				Dispose(true);
			}
		}

		protected void Dispose(bool isDisposing) 
		{
			if (isDisposing) 
			{
				GC.SuppressFinalize(this);
			}
			
			if (this.connectionProvider != null) 
			{
				((IDisposable)connectionProvider).Dispose();
			}
		}

		#endregion
	}
}
