/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/
using System;
using System.Collections;
using System.Web;
using System.Xml;
using System.Xml.Serialization;
using System.Xml.XPath;

namespace DNNContentBuilder.Configuration
{
	/// <summary>
	/// Summary description for ConfigurationBuilder.
	/// </summary>
	public class ConfigurationBuilder
	{
		private static string configPath = "~/DesktopModules/DNNContentBuilder/DnnContentBuilder.config";

		public ConfigurationBuilder()
		{
		}

		public object ReadConfiguration(string sectionName)
		{
			string fileName = HttpContext.Current.Server.MapPath(configPath);

			try {
				XmlDocument xdoc = new XmlDocument();
				xdoc.Load(fileName);

				XmlNode section = xdoc.SelectSingleNode("//" + sectionName);
				XPathNavigator navigator = section.CreateNavigator();
		
				// Get the type name to deserialize
				string typeName = (string)navigator.Evaluate("string(@type)");
			
				// TODO: handle the exception thrown if the type is not found
				Type t = Type.GetType(typeName, true);

				XmlSerializer ghost = new XmlSerializer(t);

				object graph = ghost.Deserialize(new XmlNodeReader(section));

				return graph;
			}
			catch (Exception e) {
				throw new ApplicationException(e.Message + "<P>" + fileName);
			}
		}
	}
}
