/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/
using System;
using System.Collections;
using DNNContentBuilder.Attributes;

namespace DNNContentBuilder.Configuration
{
	/// <summary>
	/// Summary description for ConfigurationManager.
	/// </summary>
	public class ConfigurationManager
	{
		private ConfigurationBuilder builder;
		private static ConfigurationManager instance;
		private static object syncObject = new object();

		private ConfigurationManager() 
		{
			this.builder = new ConfigurationBuilder();
		}

		internal static ConfigurationManager Instance
		{
			get
			{
				lock (syncObject)
				{
					if (instance == null)
					{
						instance = new ConfigurationManager();
					}

					return instance;
				}
			}
		}

		public static object GetConfiguration(Type t)
		{
			if (t.IsDefined(typeof(ConfigurationSectionAttribute), true))
			{
				object[] attributes = t.GetCustomAttributes(typeof(ConfigurationSectionAttribute), true);

				if (attributes.Length > 0)
				{
					ConfigurationSectionAttribute attr = (ConfigurationSectionAttribute)attributes[0];
					return Instance.builder.ReadConfiguration(attr.SectionName);
				}
			}

			return null;
		}

		public static object GetConfiguration(string sectionName)
		{
			return Instance.builder.ReadConfiguration(sectionName);
		}
	}
}
