/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.Data;

namespace DNNContentBuilder 
{
    [Serializable]
	public class ConnectionStringInfo 
	{
        
		// Entity member variables for database table epConnectionString
        
		private Int32 mConnectionStringId;
		private Int32 mDatabaseId;
		private string mTitle;
		private string mConnectionString;
        
		// Calculated Expression member variables for database table epConnectionString
        
        
		// Entity properties for database table ConnectionString
        
		public Int32 ConnectionStringId 
		{
			get { return mConnectionStringId; }
			set { mConnectionStringId = value; }
		}
        
        
		public Int32 DatabaseId 
		{
			get { return mDatabaseId; }
			set { mDatabaseId = value; }
		}

		public string Title 
		{
			get { return mTitle; }
			set { mTitle = value; }
		}

		public string ConnectionString 
		{
			get { return mConnectionString; }
			set { mConnectionString = value; }
		}
        
        
	}
    
	public class ConnectionStringController : BaseController
	{
		public ConnectionStringController() : base() {}
		public ConnectionStringController(ConnectionProvider cp) : base(cp) {}
        
		public Int32 Add(ConnectionStringInfo csInfo) 
		{
			return (Int32) base.GetDataProvider().ConnectionStringAdd(csInfo.Title, csInfo.DatabaseId, csInfo.ConnectionString);
		}
        
		public void Update(ConnectionStringInfo ConnectionStringInfo) 
		{
			base.GetDataProvider().ConnectionStringUpdate(ConnectionStringInfo.ConnectionStringId, ConnectionStringInfo.Title, ConnectionStringInfo.DatabaseId, ConnectionStringInfo.ConnectionString);
		}
        
		public void Delete(Int32 connectionStringId) 
		{
			base.GetDataProvider().ConnectionStringDelete(connectionStringId);
		}
        
		public ConnectionStringInfo GetItem(Int32 connectionStringId) 
		{
			using (IDataReader idr = base.GetDataProvider().ConnectionStringGet(connectionStringId))
			{
				ConnectionStringInfo infoObject = (ConnectionStringInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(ConnectionStringInfo));
				return infoObject;
			} 
		}

		public ConnectionStringInfo GetItemByTitle(string connectionTitle) 
		{
			using (IDataReader idr = base.GetDataProvider().ConnectionStringGetByTitle(connectionTitle.ToUpper().Trim()))
			{
				ConnectionStringInfo infoObject = (ConnectionStringInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(ConnectionStringInfo));
				return infoObject;
			} 
		}
        
		// Select query methods for database table epConnectionString
		public ArrayList GetAllConnections() 
		{
			using (IDataReader idr = base.GetDataProvider().ConnectionStringGetAllConnections())
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(ConnectionStringInfo));
				return infoList;
			} 
		}
	}
}
