/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections;
using System.Data;

namespace DNNContentBuilder 
{
	[Serializable]
	public class ContentViewerSettingsInfo 
	{
        
		// Entity member variables for database table epContentViewerSettings
		private Int32 mContentViewerSettingsId;
		private Int32 mModuleID;
		private Int32 mTemplateDefinitionId;
        
		// Calculated Expression member variables for database table epContentViewerSettings
        
		// Entity properties for database table epContentViewerSettings
		public Int32 ContentViewerSettingsId 
		{
			get { return mContentViewerSettingsId; }
			set { mContentViewerSettingsId = value; }
		}
        
		public Int32 ModuleID 
		{
			get { return mModuleID; }
			set { mModuleID = value; }
		}
        
		public Int32 TemplateDefinitionId 
		{
			get { return mTemplateDefinitionId; }
			set { mTemplateDefinitionId = value; }
		}
	}
    
	public class ContentViewerSettingsController : BaseController
	{
		public ContentViewerSettingsController() : base() {}
		public ContentViewerSettingsController(ConnectionProvider cp) : base(cp) {}
        
		public Int32 Add(ContentViewerSettingsInfo ContentViewerSettingsInfo) 
		{
			return (Int32) base.GetDataProvider().ContentViewerSettingsAdd(ContentViewerSettingsInfo.ModuleID, ContentViewerSettingsInfo.TemplateDefinitionId);
		}
        
		public void Update(ContentViewerSettingsInfo ContentViewerSettingsInfo) 
		{
			base.GetDataProvider().ContentViewerSettingsUpdate(ContentViewerSettingsInfo.ContentViewerSettingsId, ContentViewerSettingsInfo.ModuleID, ContentViewerSettingsInfo.TemplateDefinitionId);
		}
        
		public void Delete(Int32 contentViewerSettingsId) 
		{
			base.GetDataProvider().ContentViewerSettingsDelete(contentViewerSettingsId);
		}
        
		public ContentViewerSettingsInfo GetItem(Int32 contentViewerSettingsId) 
		{
			using (IDataReader idr = base.GetDataProvider().ContentViewerSettingsGet(contentViewerSettingsId))
			{
				ContentViewerSettingsInfo infoObject = (ContentViewerSettingsInfo)DotNetNuke.Common.Utilities.CBO.FillObject(idr, typeof(ContentViewerSettingsInfo));
				return infoObject;
			} 
		}
        
		// Relation methods for database table epContentViewerSettings
		public ArrayList GetByTemplateDefinition(Int32 templateDefinitionId) 
		{
			using (IDataReader idr = base.GetDataProvider().ContentViewerSettingsByTemplateDefinition(templateDefinitionId))
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(ContentViewerSettingsInfo));
				return infoList;
			} 
		}
        
		public ArrayList GetByModule(Int32 moduleID) 
		{
			using (IDataReader idr = base.GetDataProvider().ContentViewerSettingsByModules(moduleID))
			{
				ArrayList infoList = DotNetNuke.Common.Utilities.CBO.FillCollection(idr, typeof(ContentViewerSettingsInfo));
				return infoList;
			} 
		}
	}
}
