/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2007
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Data;

namespace DNNContentBuilder.UI.WebControls.DataBinding
{
	/// <summary>
	/// Custom data binding model class optimized for use with 
	/// <see cref="System.Data.DataTable"/> data sources.
	/// </summary>
	public class DataTableBindingModel : AbstractBindingModel
	{
		private DataTable _table;
		private string[] _columns;

		public DataTableBindingModel(DataTable table)
		{
			_table = table;
		}

		public override int Count
		{
			get
			{
				return _table != null ? _table.Rows.Count : 0;
			}
		}

		public override string Name
		{
			get
			{
				return _table != null ? _table.TableName : "NewBindingModel";
			}
		}

		public override string[] GetColumns()
		{
			if (_columns == null)
			{
				_columns = new string[_table.Columns.Count];

				for (int i = 0; i < _table.Columns.Count; i++)
				{
					_columns[i] = _table.Columns[i].ColumnName;
				}
			}

			return _columns;
		}

		/// <summary>
		/// Does nothing if there are no rows to render
		/// </summary>
		/// <param name="writer"></param>
		public override void WriteXml(System.IO.TextWriter writer)
		{
			if (_table == null || this.Count == 0)
			{
				return;
			}

			DataSet ds = new DataSet();

			ds.Tables.Add(_table);
			ds.WriteXml(writer, XmlWriteMode.IgnoreSchema);
		}
	}
}
