/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Xml;

namespace DNNContentBuilder
{
	/// <summary>
	/// Enables access to LinkerControl-specific properties.
	/// </summary>
	public class LinkProxy
	{
		#region Member Variables
		private FieldDefinitionInfo fdInfo;
		private string linkedField = null;
		#endregion

		#region Constants
		private const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/ResourceEngine";
		#endregion

		#region Constructors
		public LinkProxy(FieldDefinitionInfo fdInfo)
		{
			if (!fdInfo.IsLinker)
			{
				throw new ApplicationException(Commons.GetLocalizedString("LINK_PROXY_CANNOT_BIND", resource));
			}

			this.fdInfo = fdInfo;
		}
		#endregion

		#region Properties
		/// <summary>
		/// Gets the name of the linked field
		/// </summary>
		public string LinkedField
		{
			get
			{
				if (linkedField == null)
				{
					linkedField = this.GetPropertyValue("LinkedField");
				}

				return linkedField;
			}
		}
		#endregion

		#region Implementation
		public bool IsMatch(string columnName)
		{
			ArgumentValidator.CheckForNullReference(columnName, "columnName");
			return GetShortColumnName(columnName) == GetShortColumnName(this.LinkedField);
		}

		private string GetShortColumnName(string columnName)
		{
			ArgumentValidator.CheckForNullReference(columnName, "columnName");
			return (columnName.IndexOf('.') > -1) ? columnName.Split('.')[1] : columnName;
		}

		private string GetPropertyValue(string propertyName)
		{
			XmlDocument xd = new XmlDocument();
			xd.LoadXml(fdInfo.XML);

			XmlNode node = xd.SelectSingleNode(String.Format("//{0}", propertyName));

			if (node != null)
			{
				return node.InnerText;
			}

			return String.Empty;
		}
		#endregion
	}
}
