/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using DNNContentBuilder.UserControls.SearchFieldControls;

namespace DNNContentBuilder.UserControls.FieldControls
{
	using System;
	using System.Data;
	using System.Data.OleDb;
	using System.Xml;
	using DNNContentBuilder.Attributes;

	/// <summary>
	///	Summary description for DropDownControl.
	/// </summary>
	[FormEngineSearchControl(typeof(DropDownSearchControl))]
	public class DropDownControl : DNNContentBuilder.UserControls.FieldControls.BaseFieldControl, IFormlet
	{
		public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/DropDownControl";

		private bool isValid = true;

		private XmlDocument xmlDoc = new XmlDocument();
		private XmlNode xmlNode;

		protected System.Web.UI.WebControls.DropDownList ddList;

		private void Page_Load(object sender, System.EventArgs e)
		{
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		#region IFormlet Members

		public bool IsValid
		{
			get
			{
				return this.isValid;
			}
		}

		public string GetValue(string XML)
		{
			xmlDoc.LoadXml(XML);

			xmlNode = xmlDoc.SelectSingleNode("//SystemType");
			xmlNode.InnerText = "String";

			xmlNode = xmlDoc.SelectSingleNode("//Value");

			xmlNode.InnerText = ddList.SelectedValue;
			
			string selectedValue = xmlNode.InnerText;

			xmlNode = xmlDoc.SelectSingleNode("//IsRequired");
			
			if (xmlNode.InnerText.ToLower() == "true") {
				if (selectedValue == "" || selectedValue == "__ALL__") {
					this.isValid = false;
				}
			}			
			
			return xmlDoc.InnerXml;
		}

		public void SetValue(string XML)
		{
			xmlDoc.LoadXml(XML);
			xmlNode = xmlDoc.SelectSingleNode("//Value");

			if (xmlNode.InnerText != "") {
	            ddList.SelectedValue = xmlNode.InnerText;
			}
		}

		public void SetConfig(string XML) 
		{
			xmlDoc.LoadXml(XML);

			xmlNode = xmlDoc.SelectSingleNode("//DropDown/DataConnectionId");

			ddList.SelectedIndex = 0;

			DataConnectionController dcController = new DataConnectionController();

			DataConnectionInfo dcInfo = (DataConnectionInfo)dcController.GetItem(Int32.Parse(xmlNode.InnerText));

			ConnectionStringController csController = new ConnectionStringController();

			DatabaseController dbController = new DatabaseController();

			ConnectionStringInfo csInfo = (ConnectionStringInfo)csController.GetItem(dcInfo.ConnectionStringId);

			DatabaseInfo dbInfo = (DatabaseInfo)dbController.GetItem(csInfo.DatabaseId);

			OleDbConnection tableSourceConnection = new OleDbConnection(csInfo.ConnectionString);

			string sql = "SELECT ";
 
			xmlNode = xmlDoc.SelectSingleNode("//OptionText");

			string optionText = xmlNode.InnerText;

			sql = sql + xmlNode.InnerText + ", ";

			xmlNode = xmlDoc.SelectSingleNode("//OptionValue");

			string optionValue = xmlNode.InnerText;

			sql = sql + xmlNode.InnerText + " FROM " + dcInfo.TableName;

			xmlNode = xmlDoc.SelectSingleNode("//Where");

			if (xmlNode.InnerText != "") 
			{
				sql = sql + " WHERE " + xmlNode.InnerText;
			}

			xmlNode = xmlDoc.SelectSingleNode("//OrderBy");

			if (xmlNode.InnerText != "") 
			{
				sql = sql + " ORDER BY " + xmlNode.InnerText;
			}
			
			OleDbCommand tableSourceCommand = new OleDbCommand(sql, tableSourceConnection);

			OleDbDataReader tableSourceReader;

			try 
			{
				tableSourceConnection.Open();

				tableSourceReader = tableSourceCommand.ExecuteReader(CommandBehavior.CloseConnection);
			}
			catch (Exception error) 
			{
				string x = error.Message; //just for debug
				return;
			}

			this.ddList.DataSource = tableSourceReader;
			this.ddList.DataTextField = optionText.Replace(dcInfo.TableName+".","");
			this.ddList.DataValueField = optionValue.Replace(dcInfo.TableName+".","");
			this.ddList.DataBind();
			
			xmlNode = xmlDoc.SelectSingleNode("//CssClass");

			ddList.CssClass = xmlNode.InnerText;
		}

		#endregion

	}
}
