/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Text;
using System.Collections;
using DotNetNuke;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Portals;
using System.Web.UI;

namespace DNNContentBuilder.UserControls
{
	/// <summary>
	/// Summary description for ScheduleContent.
	/// </summary>
	public class MetaDataEvents : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.DataList dlMetaDataEventsList;
		protected System.Web.UI.WebControls.LinkButton btClose;
		protected System.Web.UI.WebControls.Literal ltEventsTable;
		
		public string lblTitle;
		public string lblEventType;
		public string lblDescription;
		public string lblDetails;
		public string lblEventDate;
		public string lblUser;
		public StringBuilder eventsTable = new StringBuilder();
		private string resource;
		private string parameters = "";
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!Request.IsAuthenticated) {
				throw new Exception("Not authenticated");
			}
			
			if (Request.Params["pid"] == "") {
				return;
			}

			parameters = (string)DataCache.GetCache(Request.Params["pid"]);
			
			if (parameters == null) {
				ltEventsTable.Text = "Metadata?";
				return;
			}
			
			int portalID = int.Parse(parameters.Split('\\')[0]);
			
			resource = Request.Params["resource"];
		
			lblTitle				= Commons.GetLocalizedString("label_Title", resource);
			lblEventType			= Commons.GetLocalizedString("label_EventType", resource);
			lblDescription    		= Commons.GetLocalizedString("label_Description", resource);
			lblDetails    			= Commons.GetLocalizedString("label_Details", resource);
			lblEventDate 			= Commons.GetLocalizedString("label_EventDate", resource);
			lblUser 				= Commons.GetLocalizedString("label_User", resource);
			btClose.Text     		= Commons.GetLocalizedString("button_Close", resource);
			
			BuildEventsTable();
						                                                        
		}

		private void BuildEventsTable() {
			MetaDataEventController meController = new MetaDataEventController();
			
			ArrayList meList = new ArrayList();
		
			try {
				meList = meController.GetEventsByMetaData(int.Parse(parameters.Split('\\')[1]));
			
				if (meList.Count > 0) {
					foreach (MetaDataEventInfo meInfo in meList) {
						eventsTable.Append("<tr>\n");
						eventsTable.Append("<td align=center>" + meInfo.EventType + "</td>\n");
						eventsTable.Append("<td>" + meInfo.Description + "</td>\n");
						eventsTable.Append("<td>" + meInfo.Details + "</td>\n");
						eventsTable.Append("<td align=center>" + meInfo.EventDate + "</td>\n");
						eventsTable.Append("<td>" + meInfo.Username + "</td>\n");
						eventsTable.Append("</tr>\n");
					}
				}
				ltEventsTable.Text = eventsTable.ToString();
			}
						
			catch (Exception ) {
				ltEventsTable.Text = "Metadata??";
			}

			
		}
		
		private void btClose_Click(object sender, System.EventArgs e)
		{
			CloseThis();
		}
		
		private void CloseThis() {
			string key = String.Format("{0}Startup", this.GetType().FullName);
            ClientScriptManager cs = Page.ClientScript;
			string script = @"
				<script language=""javascript"">
					window.close();
				</script>";

            if (!cs.IsStartupScriptRegistered(key))
			{
                cs.RegisterStartupScript(Page.GetType(),key, script);
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.btClose.Click += new System.EventHandler(this.btClose_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
