/*
 DNN Content Builder- sourceforge.net/projects/dnncb
 Copyright (c) 2004-2008
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System.Collections;
using System.ComponentModel;

namespace DNNContentBuilder.UserControls
{
	using System;

	/// <summary>
	///		Summary description for TaxonomyTreeView.
	/// </summary>
	public class TaxonomyTreeView : System.Web.UI.UserControl
	{
		#region Controls
		protected FunkeLab.MyTreeView TreeView;
		#endregion

		#region Member Variables
		private int selectedNode = -1;
		#endregion

		#region Events
		[Browsable(true)]
		public event FunkeLab.MyTreeView.OnNodeClickEventHandler NodeClick;
		#endregion

		#region Properties
		/// <summary>
		/// Gets or sets the selected node in the taxonomies tree view
		/// </summary>
		[Category("Behavior")]
		[DefaultValue(-1)]
		public int SelectedNode
		{
			get
			{
				return selectedNode;
			}
			set
			{
				selectedNode = value;
			}
		}

		/// <summary>
		/// Gets or sets whether auto postback behavior is enabled.
		/// </summary>
		[Category("Behavior")]
		[DefaultValue(false)]
		public bool AutoPostBack
		{
			get
			{
				return this.TreeView.AutoPostBack;
			}
			set
			{
				this.TreeView.AutoPostBack = true;
			}
		}
		#endregion

		#region Event Handlers
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!this.IsPostBack)
			{
				BuildTree();
			}
		}

		protected virtual void OnNodeClick(ref FunkeLab.MyTreeViewItem item)
		{
			if (this.NodeClick != null)
			{
				this.NodeClick(ref item);
			}
		}
		#endregion

		#region Methods
		public void BuildTree() 
		{
			string path = this.ResolveUrl("~/DeskTopModules/DNNContentBuilder/images/");
			FunkeLab.MyTreeView.ImagesFolder = path;

			TreeView.Items.Clear();

			FunkeLab.MyTreeViewItem item = new FunkeLab.MyTreeViewItem();
		
			item.Text	  = "Root";
			item.Selector = FunkeLab.MyTreeViewItem.SelectionMode.RadioButton;
			item.Selected = true;
			item.Value	  = "-1";
			
			TreeView.Items.Add(this.CreateTreeNodes(-1, item));

			TreeView.OpenAll();
		}

		private FunkeLab.MyTreeViewItem CreateTreeNodes(int node, FunkeLab.MyTreeViewItem localItem)
		{
			ArrayList list = new TaxonomyController().GetTaxonomySubTree(node);

			foreach (TaxonomyInfo taxInfo in list)  
			{
				// Create treeview item
				FunkeLab.MyTreeViewItem item = new FunkeLab.MyTreeViewItem();

				item.Text  = taxInfo.Title;
				item.Value = taxInfo.TaxonomyId.ToString();

				if (Int32.Parse(item.Value) == SelectedNode) 
				{
					item.Selected = true;
				} 
				else 
				{
					item.Selected = false;
				}

				item.Selector = FunkeLab.MyTreeViewItem.SelectionMode.RadioButton;

				localItem.Items.Add(CreateTreeNodes(Int32.Parse(item.Value), item));
			}

			return localItem;
		}
		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);
			this.TreeView.OnNodeClick += new FunkeLab.MyTreeView.OnNodeClickEventHandler(OnNodeClick);
		}
		#endregion
	}
}
